@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -72,6 +72,7 @@
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/module.h"
+#include "magick/utility.h"
 
 /*
   Typedef declarations.
@@ -333,6 +334,7 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Determine if this a PDB image file.
   */
+  (void) memset(&pdb_info,0,sizeof(pdb_info));
   count=ReadBlob(image,sizeof(pdb_info.name),(unsigned char *) pdb_info.name);
   if (count != sizeof(pdb_info.name))
     ThrowReaderException(CorruptImageError,"ImproperImageHeader");
@@ -414,6 +416,8 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       return(GetFirstImageInList(image));
     }
   status=SetImageExtent(image,image->columns,image->rows);
+  if (status != MagickFalse)
+    status=ResetImagePixels(image,exception);
   if (status == MagickFalse)
     {
       InheritException(exception,&image->exception);
@@ -424,18 +428,24 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     sizeof(*pixels));
   if (pixels == (unsigned char *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  (void) memset(pixels,0,(packets+257UL)*image->rows*sizeof(*pixels));
   switch (pdb_image.version & 0x07)
   {
     case 0:
     {
       image->compression=NoCompression;
       count=(ssize_t) ReadBlob(image,packets*image->rows,pixels);
+      if (count != (ssize_t) (packets*image->rows))
+        {
+          pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+          ThrowReaderException( CorruptImageError,"RLEDecoderError");
+        }
       break;
     }
     case 1:
     {
       image->compression=RLECompression;
-      if (!DecodeImage(image,pixels,packets*image->rows))
+      if (DecodeImage(image,pixels,packets*image->rows) == MagickFalse)
         {
           pixels=(unsigned char *) RelinquishMagickMemory(pixels);
           ThrowReaderException( CorruptImageError,"RLEDecoderError");
@@ -443,11 +453,11 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
       break;
     }
     default:
-      {
-        pixels=(unsigned char *) RelinquishMagickMemory(pixels);
-        ThrowReaderException(CorruptImageError,
-          "UnrecognizedImageCompressionType");
-      }
+    {
+      pixels=(unsigned char *) RelinquishMagickMemory(pixels);
+      ThrowReaderException(CorruptImageError,
+        "UnrecognizedImageCompressionType");
+    }
   }
   p=pixels;
   switch (bits_per_pixel)
@@ -466,14 +476,18 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (q == (PixelPacket *) NULL)
           break;
         indexes=GetAuthenticIndexQueue(image);
-        for (x=0; x < ((ssize_t) image->columns-7); x+=8)
+        bit=0;
+        for (x=0; x < (ssize_t) image->columns; x++)
         {
-          for (bit=0; bit < 8; bit++)
-          {
-            index=(IndexPacket) (*p & (0x80 >> bit) ? 0x00 : 0x01);
-            SetPixelIndex(indexes+x+bit,index);
-          }
-          p++;
+          index=(Quantum) (*p & (0x80 >> bit) ? 0x00 : 0x01);
+          indexes[x]=index;
+          q++;
+          bit++;
+          if (bit == 8)
+            {
+              p++;
+              bit=0;
+            }
         }
         if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
@@ -487,6 +501,9 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
     case 2:
     {
+      unsigned int
+        shift;
+
       /*
         Read 2-bit PDB image.
       */
@@ -496,17 +513,17 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (q == (PixelPacket *) NULL)
           break;
         indexes=GetAuthenticIndexQueue(image);
-        for (x=0; x < (ssize_t) image->columns-3; x+=4)
+        shift=8;
+        for (x=0; x < (ssize_t) image->columns; x++)
         {
-          index=ConstrainColormapIndex(image,3UL-((*p >> 6) & 0x03));
+          shift-=2;
+          index=ConstrainColormapIndex(image,3UL-((*p >> shift) & 0x03));
           SetPixelIndex(indexes+x,index);
-          index=ConstrainColormapIndex(image,3UL-((*p >> 4) & 0x03));
-          SetPixelIndex(indexes+x+1,index);
-          index=ConstrainColormapIndex(image,3UL-((*p >> 2) & 0x03));
-          SetPixelIndex(indexes+x+2,index);
-          index=ConstrainColormapIndex(image,3UL-((*p) & 0x03));
-          SetPixelIndex(indexes+x+3,index);
-          p++;
+          if (shift == 0)
+            {
+              shift=8;
+              p++;
+            }
         }
         if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
@@ -520,6 +537,9 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
     case 4:
     {
+      unsigned int
+        shift;
+
       /*
         Read 4-bit PDB image.
       */
@@ -529,13 +549,17 @@ static Image *ReadPDBImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (q == (PixelPacket *) NULL)
           break;
         indexes=GetAuthenticIndexQueue(image);
-        for (x=0; x < (ssize_t) image->columns-1; x+=2)
+        shift=8;
+        for (x=0; x < (ssize_t) image->columns; x++)
         {
-          index=ConstrainColormapIndex(image,15UL-((*p >> 4) & 0x0f));
+          shift-=4;
+          index=ConstrainColormapIndex(image,15UL-((*p >> shift) & 0x0f));
           SetPixelIndex(indexes+x,index);
-          index=ConstrainColormapIndex(image,15UL-((*p) & 0x0f));
-          SetPixelIndex(indexes+x+1,index);
-          p++;
+          if (shift == 0)
+            {
+              shift=8;
+              p++;
+            }
         }
         if (SyncAuthenticPixels(image,exception) == MagickFalse)
           break;
@@ -707,7 +731,7 @@ static unsigned char *EncodeRLE(unsigned char *destination,
 {
   if (literal > 0)
     *destination++=(unsigned char) (literal-1);
-  (void) CopyMagickMemory(destination,source,literal);
+  (void) memcpy(destination,source,literal);
   destination+=literal;
   if (repeat > 0)
     {
@@ -719,6 +743,9 @@ static unsigned char *EncodeRLE(unsigned char *destination,
 
 static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
 {
+  char
+    filename[MaxTextExtent];
+
   const char
     *comment;
 
@@ -784,9 +811,10 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   } else {
     bits_per_pixel=4;
   }
-  (void) ResetMagickMemory(&pdb_info,0,sizeof(pdb_info));
-  (void) CopyMagickString(pdb_info.name,image_info->filename,
-    sizeof(pdb_info.name));
+  (void) memset(&pdb_info,0,sizeof(pdb_info));
+  (void) memset(&pdb_image,0,sizeof(pdb_image));
+  GetPathComponent(image_info->filename,TailPath,filename);
+  (void) CopyMagickString(pdb_info.name,filename,sizeof(pdb_info.name));
   pdb_info.attributes=0;
   pdb_info.version=0;
   pdb_info.create_time=time(NULL);
@@ -795,8 +823,8 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   pdb_info.modify_number=0;
   pdb_info.application_info=0;
   pdb_info.sort_info=0;
-  (void) CopyMagickMemory(pdb_info.type,"vIMG",4);
-  (void) CopyMagickMemory(pdb_info.id,"View",4);
+  (void) memcpy(pdb_info.type,"vIMG",4);
+  (void) memcpy(pdb_info.id,"View",4);
   pdb_info.seed=0;
   pdb_info.next_record=0;
   comment=GetImageProperty(image,"comment");
@@ -835,7 +863,7 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
     pdb_image.width=(short) (16*(image->columns/16+1));
   pdb_image.height=(short) image->rows;
   packets=((bits_per_pixel*image->columns+7)/8);
-  packet_size=(size_t) (image->depth > 8 ? 2 : 1);
+  packet_size=(size_t) (bits_per_pixel > 8 ? 2 : 1);
   runlength=(unsigned char *) AcquireQuantumMemory(9UL*packets,
     image->rows*sizeof(*runlength));
   buffer=(unsigned char *) AcquireQuantumMemory(512,sizeof(*buffer));
@@ -853,6 +881,7 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
         scanline=(unsigned char *) RelinquishMagickMemory(scanline);
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     }
+  (void) ResetMagickMemory(buffer,0,512*sizeof(*buffer));
   if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
     (void) TransformImageColorspace(image,sRGBColorspace);
   /*
@@ -860,8 +889,16 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   */
   quantum_info=AcquireQuantumInfo(image_info,image);
   if (quantum_info == (QuantumInfo *) NULL)
-    ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-  status=SetQuantumDepth(image,quantum_info,image->depth > 8 ? 16 : 8);
+    {
+      if (runlength != (unsigned char *) NULL)
+        runlength=(unsigned char *) RelinquishMagickMemory(runlength);
+      if (buffer != (unsigned char *) NULL)
+        buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+      if (scanline != (unsigned char *) NULL)
+        scanline=(unsigned char *) RelinquishMagickMemory(scanline);
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+    }
+  status=SetQuantumDepth(image,quantum_info,bits_per_pixel > 8 ? 16 : 8);
   bits=8/(int) bits_per_pixel-1;  /* start at most significant bits */
   literal=0;
   repeat=0;
@@ -913,7 +950,7 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
               if (0x7f < literal)
                 {
                   q=EncodeRLE(q,buffer,(literal < 0x80 ? literal : 0x80),0);
-                  (void) CopyMagickMemory(buffer,buffer+literal+repeat,0x80);
+                  (void) memmove(buffer,buffer+literal+repeat,0x80);
                   literal-=0x80;
                 }
             }
